@@ -1861,8 +1861,13 @@ impl App {
         )?;
 
         // Make the reporter.
-        let mut reporter =
-            reporter_builder.build(&test_list, &profile, output, structured_reporter);
+        let mut reporter = reporter_builder.build(
+            &test_list,
+            &profile,
+            &self.base.cargo_configs,
+            output,
+            structured_reporter,
+        );
 
         configure_handle_inheritance(no_capture)?;
         let run_stats = runner.try_execute(|event| {
@@ -2228,7 +2233,7 @@ impl DebugCommand {
                 let build_platforms = detect_build_platforms(&cargo_configs, target.as_deref())?;
                 match output_format {
                     BuildPlatformsOutputFormat::Debug => {
-                        println!("{:#?}", build_platforms);
+                        println!("{build_platforms:#?}");
                     }
                     BuildPlatformsOutputFormat::Triple => {
                         println!(
@@ -2454,7 +2459,7 @@ fn log_platform_runner(prefix: &str, runner: &PlatformRunner, styles: &StderrSty
     )
 }
 
-fn warn_on_err(thing: &str, err: &(dyn std::error::Error), styles: &StderrStyles) {
+fn warn_on_err(thing: &str, err: &dyn std::error::Error, styles: &StderrStyles) {
     let mut s = String::with_capacity(256);
     swrite!(s, "could not determine {thing}: {err}");
     let mut next_error = err.source();
