@@ -269,6 +269,9 @@ pub enum TestEventKind<'a> {
         /// The number of setup scripts still running.
         setup_scripts_running: usize,
 
+        /// Current statistics for number of tests so far.
+        current_stats: RunStats,
+
         /// The number of tests still running.
         running: usize,
 
@@ -281,6 +284,9 @@ pub enum TestEventKind<'a> {
         /// The number of setup scripts still running.
         setup_scripts_running: usize,
 
+        /// Current statistics for number of tests so far.
+        current_stats: RunStats,
+
         /// The number of tests still running.
         running: usize,
 
@@ -875,9 +881,9 @@ impl fmt::Debug for AbortStatus {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             #[cfg(unix)]
-            AbortStatus::UnixSignal(signal) => write!(f, "UnixSignal({})", signal),
+            AbortStatus::UnixSignal(signal) => write!(f, "UnixSignal({signal})"),
             #[cfg(windows)]
-            AbortStatus::WindowsNtStatus(status) => write!(f, "WindowsNtStatus({:x})", status),
+            AbortStatus::WindowsNtStatus(status) => write!(f, "WindowsNtStatus({status:x})"),
             #[cfg(windows)]
             AbortStatus::JobObject => write!(f, "JobObject"),
         }
@@ -898,6 +904,9 @@ pub enum CancelReason {
     /// An error occurred while reporting results.
     ReportError,
 
+    /// The global timeout was exceeded.
+    GlobalTimeout,
+
     /// A termination signal (on Unix, SIGTERM or SIGHUP) was received.
     Signal,
 
@@ -914,6 +923,7 @@ impl CancelReason {
             CancelReason::SetupScriptFailure => "setup script failure",
             CancelReason::TestFailure => "test failure",
             CancelReason::ReportError => "reporting error",
+            CancelReason::GlobalTimeout => "global timeout",
             CancelReason::Signal => "signal",
             CancelReason::Interrupt => "interrupt",
             CancelReason::SecondSignal => "second signal",
