@@ -44,6 +44,7 @@ pub(super) struct DispatcherContext<'a, F> {
     stopwatch: StopwatchStart,
     run_stats: RunStats,
     max_fail: MaxFail,
+    global_timeout: Duration,
     running_setup_script: Option<ContextSetupScript<'a>>,
     running_tests: BTreeMap<TestInstanceId<'a>, ContextTestInstance<'a>>,
     cancel_state: Option<CancelReason>,
@@ -63,6 +64,7 @@ where
         cli_args: Vec<String>,
         initial_run_count: usize,
         max_fail: MaxFail,
+        global_timeout: Duration,
     ) -> Self {
         Self {
             callback: DebugIgnore(callback),
@@ -75,6 +77,7 @@ where
                 ..RunStats::default()
             },
             max_fail,
+            global_timeout,
             running_setup_script: None,
             running_tests: BTreeMap::new(),
             cancel_state: None,
@@ -103,9 +106,14 @@ where
         let mut signals_done = false;
         let mut inputs_done = false;
         let mut report_cancel_rx_done = false;
+        let mut global_timeout_sleep =
+            std::pin::pin!(crate::time::pausable_sleep(self.global_timeout));
 
         loop {
             let internal_event = tokio::select! {
+                _ = &mut global_timeout_sleep => {
+                    InternalEvent::GlobalTimeout
+                },
                 internal_event = executor_rx.recv() => {
                     match internal_event {
                         Some(event) => InternalEvent::Executor(event),
@@ -201,6 +209,9 @@ where
                     // Restore the terminal state.
                     input_handler.suspend();
 
+                    // Pause the global timeout while suspended.
+                    global_timeout_sleep.as_mut().pause();
+
                     // Now stop nextest itself.
                     super::os::raise_stop();
                 }
@@ -208,6 +219,10 @@ where
                 HandleEventResponse::JobControl(JobControlEvent::Continue) => {
                     // Nextest has been resumed. Resume the input handler, as well as all the tests.
                     input_handler.resume();
+
+                    // Resume the global timeout.
+                    global_timeout_sleep.as_mut().resume();
+
                     self.broadcast_request(RunUnitRequest::Signal(SignalRequest::Continue));
                 }
                 #[cfg(not(unix))]
@@ -285,6 +300,14 @@ where
                             // A test failure has caused cancellation to begin.
                             self.broadcast_request(RunUnitRequest::OtherCancel);
                         }
+                        CancelEvent::GlobalTimeout => {
+                            // The global timeout has expired, causing cancellation to begin.
+                            self.broadcast_request(RunUnitRequest::Signal(
+                                SignalRequest::Shutdown(ShutdownRequest::Once(
+                                    ShutdownEvent::GLOBAL_TIMEOUT,
+                                )),
+                            ));
+                        }
                         CancelEvent::Signal(req) => {
                             // A signal has caused cancellation to begin. Let all the child
                             // processes know about the signal, and continue to handle
@@ -534,6 +557,9 @@ where
                 })
             }
             InternalEvent::Signal(event) => self.handle_signal_event(event),
+            InternalEvent::GlobalTimeout => {
+                self.begin_cancel(CancelReason::GlobalTimeout, CancelEvent::GlobalTimeout)
+            }
             InternalEvent::Input(InputEvent::Info) => {
                 // Print current statistics.
                 HandleEventResponse::Info(InfoEvent::Input)
@@ -764,6 +790,7 @@ where
             // signal.
             self.basic_callback(TestEventKind::RunBeginKill {
                 setup_scripts_running: self.setup_scripts_running(),
+                current_stats: self.run_stats,
                 running: self.running(),
                 // This is always a second signal.
                 reason: CancelReason::SecondSignal,
@@ -773,6 +800,7 @@ where
             self.cancel_state = Some(reason);
             self.basic_callback(TestEventKind::RunBeginCancel {
                 setup_scripts_running: self.setup_scripts_running(),
+                current_stats: self.run_stats,
                 running: self.running(),
                 reason,
             });
@@ -848,6 +876,7 @@ enum InternalEvent<'a> {
     Signal(SignalEvent),
     Input(InputEvent),
     ReportCancel,
+    GlobalTimeout,
 }
 
 /// The return result of `handle_event`.
@@ -881,6 +910,7 @@ enum InfoEvent {
 enum CancelEvent {
     Report,
     TestFailure,
+    GlobalTimeout,
     Signal(ShutdownRequest),
 }
 
@@ -917,7 +947,9 @@ mod tests {
             vec![],
             0,
             MaxFail::All,
+            crate::time::far_future_duration(),
         );
+
         cx.disable_signal_3_times_panic = true;
 
         // Begin cancellation with a report error.
@@ -933,6 +965,7 @@ mod tests {
             let event = events.pop().unwrap();
             let TestEventKind::RunBeginCancel {
                 setup_scripts_running,
+                current_stats: _,
                 running,
                 reason,
             } = event.kind
@@ -960,7 +993,7 @@ mod tests {
         // yet.
         for sig1 in ShutdownEvent::ALL_VARIANTS {
             for sig2 in ShutdownEvent::ALL_VARIANTS {
-                eprintln!("** testing {:?} -> {:?}", sig1, sig2);
+                eprintln!("** testing {sig1:?} -> {sig2:?}");
                 // Separate test for each signal to avoid mixing up state.
                 let mut cx = cx.clone();
 
@@ -977,6 +1010,7 @@ mod tests {
                     let event = events.pop().unwrap();
                     let TestEventKind::RunBeginCancel {
                         setup_scripts_running,
+                        current_stats: _,
                         running,
                         reason,
                     } = event.kind
@@ -1005,6 +1039,7 @@ mod tests {
                     let event = events.pop().unwrap();
                     let TestEventKind::RunBeginKill {
                         setup_scripts_running,
+                        current_stats: _,
                         running,
                         reason,
                     } = event.kind
